/*
  Beegee - Created by James Thomson

  Allows the use of multiple background images which are queried depending on viewport width.
*/

;(function(root, beegee) {
    if (typeof define === 'function' && define.amd) {
        // AMD. Register Beegee as an anonymous module
        define(beegee);
    } else if (typeof exports === 'object') {
        // Node. Does not work with strict CommonJS, but
        // only CommonJS-like environments that support module.exports,
        // like Node.
        module.exports = beegee();
    } else {
        // Browser globals. Register Beegee on window
        root.Beegee = beegee();
    }
})(this, function() {
    'use strict';

    // our constructor
    return function Beegee(settings) {
      var scope = this;
      var util = {};

      var _viewport = 0;
      var _dpr = 1;
      var _els = [];
      var _source = '';
      var _sourceDpr;
      var _resizeTimer;

      var _defaultBreakpoints = [
        { 
          minWidth: 320,
          src: 'data-bg-mobile'
        }, 
        {
          minWidth: 768,
          src: 'data-bg-desktop'
        }
      ];

      scope.settings = settings || {};
      scope.settings.selector = scope.settings.selector || '.js-beegee';
      scope.settings.dprPrefix = scope.settings.dprPrefix || '-';
      scope.settings.dprSuffix = scope.settings.dprSuffix || 'x';
      scope.settings.breakpoints = scope.settings.breakpoints || _defaultBreakpoints;     


      /* Private Functions */
      function init() {
        // Gather elements
        _els = util.selectAll(scope.settings.selector);
        _dpr = window.devicePixelRatio || 1;

        sequence();

        window.addEventListener('resize', function() {
          // Debouncing timer
          clearTimeout(_resizeTimer);
            _resizeTimer = setTimeout(function() {

              sequence();
                      
            }, 250);
        });
      }

      function sequence() {
        getViewportWidth();

        setCurrentSource();
        setBgImage();
      }

      // Stores the current viewport width
      function getViewportWidth() {
        _viewport = window.innerWidth;
      }

      // Sets the current breakpoint value
      function setCurrentSource() {
        // Loop through breakpoints
        if (scope.settings.breakpoints) {
          doEach(scope.settings.breakpoints, function(object) {
            // Determine which breakpoint to use
            if (_viewport >= object.minWidth) {
              _source = object.src;
            }
          });
        }

        // Let's set the current source device pixel ratio while we're at it
        setSourceDpr();
      }

      function setSourceDpr() {
        _sourceDpr = scope.settings.dprPrefix + _dpr + scope.settings.dprSuffix;
      }

      // Sets a background image attribute on the object
      function setBgImage() {
        doEach(_els, function(object){
          var src = object.getAttribute(_source);
          var dprRegex = new RegExp(scope.settings.dprPrefix + '1' + scope.settings.dprSuffix, 'gi');

          if (src) {
            // Check if src has dpr in it already, eg. '-1x'
            if (!src.match(dprRegex)) {
              // Construct src with dpr suffix
              src = insertDpr(src);
            }
          } else {
            console.log('There was an issue with retrieving the value for ' + _source + ' we got ' + src);
          }

          object.style.backgroundImage = 'url(' + src + ')';
        });
      }

      function insertDpr(string) {
        var newString;
        var stringExt;
        var extRegEx = /.png|.jpg|.jpeg|.gif/gi;

        // Extract the extension
        stringExt = string.match(extRegEx);

        // Rebuild the string inserting the DPR
        newString = string.replace(extRegEx, _sourceDpr + stringExt);

        return newString;
      }

      // Runs function for each object
      function doEach(object, fn) {
        if (object && fn) {
          var l = object.length;
          for (var i = 0; i < l && fn(object[i], i) !== false; i++) {}
        }
      }

      /* Utilities */
      util.select = function (string) {
        return document.querySelector(string);
      }

      util.selectAll = function (string) {
        return document.querySelectorAll(string);
      };

      // Boot it all up
      init();
    }
});
